@@ -1,15 +1,18 @@
 //! Initialises recording device and audio stream's silence level
-
+use anyhow::{anyhow, Result};
 use cpal::{
     traits::{DeviceTrait, HostTrait},
     Device, SampleRate, SupportedStreamConfig,
 };
+use log::error;
+
+const SAMPLE_RATE: u32 = 16000;
 
 ///
 /// # Input device configuration
 /// Gets data ready to begin recording
 
-pub struct StreamConfig {
+pub(crate) struct StreamConfig {
     device: Device,
     config: SupportedStreamConfig,
     silence_level: i32,
@@ -24,19 +27,24 @@ mod tests {
     fn supported_device() {
         let host = cpal::default_host();
         let device = host.default_input_device().expect("no input device found");
-
-        assert_eq!(1, get_config(&device).channels());
+        assert_eq!(1, get_config(&device).unwrap().channels());
     }
 }
 
 impl StreamConfig {
     /// Creates a new stream configuration:
-    pub fn new(silence_level: i32) -> Self {
-        let device = get_default_device();
-        StreamConfig {
-            config: get_config(&device),
-            device,
-            silence_level,
+    pub fn new(silence_level: i32) -> Result<Self> {
+        let device = get_default_device()?;
+        match get_config(&device) {
+            Ok(config) => Ok(StreamConfig {
+                config,
+                device,
+                silence_level,
+            }),
+            Err(e) => {
+                error!("{}", e);
+                Err(anyhow!(e))
+            }
         }
     }
 
@@ -57,22 +65,23 @@ impl StreamConfig {
 }
 
 ///Returns the configuration of the mono channel
-fn get_config(device: &Device) -> SupportedStreamConfig {
-    let mut config = device
-        .default_input_config()
-        .expect("Failed to get input config");
-    while config.channels() != 1 {
-        let mut supported_configs_range = device
-            .supported_input_configs()
-            .expect("error while querying configs");
+fn get_config(device: &Device) -> Result<SupportedStreamConfig> {
+    let mut config = device.default_input_config()?;
+    if config.channels() != 1 {
+        let mut supported_configs_range = device.supported_input_configs()?;
         config = match supported_configs_range.next() {
-            Some(conf) => conf.with_sample_rate(SampleRate(16000)), //16K from deepspeech
-            None => break,
+            Some(conf) => {
+                conf.with_sample_rate(SampleRate(SAMPLE_RATE)) //16K from deepspeech
+            }
+            None => config,
         };
     }
-    config
+    Ok(config)
 }
-fn get_default_device() -> Device {
+fn get_default_device() -> Result<Device> {
     let host = cpal::default_host();
-    host.default_input_device().expect("no input device found")
+    match host.default_input_device() {
+        Some(device) => Ok(device),
+        None => Err(anyhow!("no input device found")),
+    }
 }
