@@ -1,16 +1,19 @@
 //! Does the actual recording and white noise cleaning
 
+use crate::config::StreamConfig;
+use anyhow::Result;
+use cpal::traits::{DeviceTrait, StreamTrait};
+use log::error;
+#[cfg(feature = "denoise")]
+use log::trace;
+#[cfg(feature = "denoise")]
+use nnnoiseless::DenoiseState;
 use std::{
     ops::Neg,
     sync::mpsc::{channel, Receiver},
     time::Instant,
 };
 
-use cpal::traits::{DeviceTrait, StreamTrait};
-use nnnoiseless::DenoiseState;
-
-use crate::config::StreamConfig;
-
 ///
 /// # Recording Audio
 /// This function handles the audio recording process, parameters are obtained from `ds_transcriber::transcriber::StreamSettings`
@@ -20,42 +23,38 @@ pub fn record_audio(
     silence_level: i32,
     show_amplitude: bool,
     pause_length: f32,
-) -> Option<Vec<i16>> {
-    let config = StreamConfig::new(silence_level);
+) -> Result<Vec<i16>> {
+    let config = StreamConfig::new(silence_level)?;
     let (sound_sender, sound_receiver) = channel();
     let device = config.device();
     let stream_config = config.supported_config().config();
-    let stream = device
-        .build_input_stream(
-            &stream_config,
-            move |data: &[f32], _: &_| {
-                sound_sender.send(data.to_owned()).unwrap();
-            },
-            move |err| println!("Stream read error: {}", err),
-        )
-        .expect("Failed to process stream");
+    let stream = device.build_input_stream(
+        &stream_config,
+        move |data: &[f32], _: &_| {
+            if let Err(e) = sound_sender.send(data.to_owned()) {
+                error!("{}", e);
+            }
+        },
+        move |err| error!("Stream read error: {}", err),
+    )?;
 
     match stream.play() {
         Ok(()) => {
-            let denoised_stream = Some(start(
+            let denoised_stream = start(
                 &sound_receiver,
                 config.silence_level(),
                 show_amplitude,
                 pause_length,
-            ))
-            .unwrap()
-            .unwrap();
-
-            let mut audio_buf: Vec<_> = Vec::new();
-            //convert to i16 stream
-            for s in denoised_stream {
-                audio_buf.push((s * i16::max_value() as f32) as i16);
-            }
-            Some(audio_buf)
+            )?;
+            let audio_buf = denoised_stream
+                .into_iter()
+                .map(|a| (a * i16::MAX as f32) as i16)
+                .collect::<Vec<_>>();
+            Ok(audio_buf)
         }
         Err(err) => {
-            eprintln!("Failed to start the stream: {}", err);
-            None
+            error!("Failed to start the stream: {}", err);
+            Err(anyhow::anyhow!(err))
         }
     }
 }
@@ -65,11 +64,11 @@ fn start(
     silence_level: i32,
     show_amplitude: bool,
     pause_length: f32,
-) -> Option<Vec<f32>> {
+) -> Result<Vec<f32>> {
     let mut silence_start = None;
     let mut sound_from_start_till_pause = vec![];
     loop {
-        let small_sound_chunk = sound_receiver.recv().unwrap();
+        let small_sound_chunk = sound_receiver.recv()?;
         sound_from_start_till_pause.extend(&small_sound_chunk);
         let sound_as_ints = small_sound_chunk.iter().map(|f| (*f * 1000.0) as i32);
         let max_amplitude = sound_as_ints.clone().max().unwrap_or(0);
@@ -83,7 +82,12 @@ fn start(
                 None => silence_start = Some(Instant::now()),
                 Some(s) => {
                     if s.elapsed().as_secs_f32() > pause_length {
-                        return Some(denoise(sound_from_start_till_pause));
+                        #[cfg(feature = "denoise")]
+                        {
+                            trace!("denoising stream");
+                            return Ok(denoise(sound_from_start_till_pause));
+                        }
+                        return Ok(sound_from_start_till_pause);
                     }
                 }
             }
@@ -93,6 +97,7 @@ fn start(
     }
 }
 
+#[cfg(feature = "denoise")]
 fn denoise(sound_from_start_till_pause: Vec<f32>) -> Vec<f32> {
     let mut output = Vec::new();
     let mut out_buf = [0.0; DenoiseState::FRAME_SIZE];
@@ -100,7 +105,6 @@ fn denoise(sound_from_start_till_pause: Vec<f32>) -> Vec<f32> {
     let mut first = true;
     for chunk in sound_from_start_till_pause.chunks_exact(DenoiseState::FRAME_SIZE) {
         denoise.process_frame(&mut out_buf[..], chunk);
-
         if !first {
             output.extend_from_slice(&out_buf[..]);
         }
